class ExercisePublishTask
  include ExercisesHelper
  include GitHelper

  def publish
    Rails.logger.info("log--------------------------------exercise_publish start")
    puts "--------------------------------exercise_publish start"
    exercises = Exercise.includes(:exercise_users).where("publish_time is not null and exercise_status = 1 and publish_time <=?",Time.now)
    exercises&.each do |exercise|
      exercise.update_column('exercise_status', 2)

      if exercise.unified_setting
        ExercisePublishNotifyJob.perform_later(exercise.id, nil)
      else
        course = exercise.course
        teachers = course.teachers.where.not(id: course.teacher_course_groups.select(:course_member_id))
        tid_str = ""
        teachers.find_each do |member|
          tid_str += "," if tid_str != ""
          tid_str += "(#{member.user_id}, #{exercise.user_id}, #{exercise.id}, 'Exercise', #{exercise.id}, 'ExercisePublish', #{course.id}, 'Course', 0, 'Exercise', '#{Time.now.strftime('%Y-%m-%d %H:%M:%S')}', '#{Time.now.strftime('%Y-%m-%d %H:%M:%S')}')"
        end
        if tid_str != ""
          tid_sql = "insert into tidings (user_id, trigger_user_id, container_id, container_type, parent_container_id, parent_container_type, belong_container_id, belong_container_type, viewed, tiding_type, created_at, updated_at) values" + tid_str
          ActiveRecord::Base.connection.execute tid_sql
        end
      end

      exercise.course_acts << CourseActivity.new(user_id: exercise.user_id, course_id: exercise.course_id) if !exercise.course_acts.exists?
    end

    # 分组设置发布时间的测验
    exercise_group_settings = ExerciseGroupSetting.where("publish_time < ? and publish_time > ?", Time.now + 900, Time.now - 900)
    exercise_group_settings&.each do |exercise_group|
      exercise = exercise_group.exercise
      if exercise.present?
        exercise.update_attributes(:exercise_status => 2) if exercise.exercise_status == 1
        ExercisePublishNotifyJob.perform_later(exercise.id, [exercise_group.course_group_id])
      end
    end
    Rails.logger.info("log--------------------------------exercise_publish end")
    puts "--------------------------------exercise_publish end"
  end

  def end
    Rails.logger.info("log--------------------------------exercise_end start")
    puts "--------------------------------exercise_end start"
    # 1。统一设置的试卷
    exercises = Exercise.includes(:exercise_users,:exercise_questions).where("exercise_status = 2 AND unified_setting = true AND end_time <= ?",Time.now + 900)
    exercises&.each do |exercise|
      ex_type = exercise.exercise_questions.pluck(:question_type).uniq
      exercise.update_column('exercise_status', 3)
      exercise.exercise_users&.each do |exercise_user|
        begin
          if (exercise_user&.commit_status == 0) && (exercise_user&.start_at.present?)
            s_score = calculate_student_score(exercise, exercise_user.user)[:total_score]
            if ex_type.include?(4)   #是否包含主观题
              subjective_score = exercise_user.subjective_score
            else
              subjective_score = -1.0
            end
            total_score_subjective_score = subjective_score < 0.0 ? 0.0 : subjective_score
            total_score = s_score + total_score_subjective_score
            commit_option = {
                :status => 1,
                :commit_status => 1,
                :end_at => Time.now,
                :objective_score => s_score,
                :score => total_score,
                :subjective_score => subjective_score
            }
            exercise_user.update_attributes(commit_option)
          end
        rescue Exception => e
          Rails.logger.info("rescue errors ___________________________#{e}")
          next
        end

      end
    end

    # 2.非统一的试卷
    all_exercises = Exercise.includes(:exercise_group_settings,:exercise_users,:exercise_questions).where("unified_setting = false AND exercise_status = 2 AND end_time > ?",Time.now + 900)
    exercise_ids = all_exercises.blank? ? "(-1)" : "(" + all_exercises.map(&:id).join(",") + ")"
    ex_group_settings = ExerciseGroupSetting.where("end_time <= '#{Time.now}' and exercise_id in #{exercise_ids}")
    ex_group_settings&.each do |exercise_setting|
      exercise = exercise_setting.exercise
      if exercise.end_time <= Time.now
        exercise.update_column('exercise_status', 3)
      end

      ex_types = exercise.exercise_questions.pluck(:question_type).uniq
      users = exercise.course.students.where(:course_group_id => exercise_setting.course_group_id)
      exercise_users = exercise.exercise_users.where(:user_id => users.pluck(:user_id))
      exercise_users&.each do |exercise_user|
        begin
          if exercise_user.commit_status == 0 && !exercise_user.start_at.nil?
            if ex_types.include?(4)   #是否包含主观题
              subjective_score = exercise_user.subjective_score
            else
              subjective_score = -1.0
            end
            s_score = calculate_student_score(exercise, exercise_user.user)[:total_score]
            total_score_subjective_score = subjective_score < 0.0 ? 0.0 : subjective_score
            total_score = s_score + total_score_subjective_score
            commit_option = {
                :status => 1,
                :commit_status => 1,
                :end_at => Time.now,
                :objective_score => s_score,
                :score => total_score,
                :subjective_score => subjective_score
            }
            exercise_user.update_attributes(commit_option)
          end
        rescue Exception => e
          Rails.logger.info("unified_setting_false_rescue errors ___________________________#{e}")
          next
        end

      end
    end
    Rails.logger.info("log--------------------------------exercise_end end")
    puts "--------------------------------exercise_end end"
  end

end